import bpy
from . import image_util
from .base_functions import TF_base_functions, load_image
from os.path import isfile, exists
from os import makedirs
from mathutils import Vector, Euler
from math import pi

def set_render_scene(render_scene):
	render_scene.render.resolution_y = 96*4
	render_scene.render.resolution_x = 128
	render_scene.render.resolution_percentage = 100
	render_scene.render.use_antialiasing = True
	render_scene.render.antialiasing_samples = '5'
	render_scene.render.alpha_mode = 'TRANSPARENT'
	render_scene.render.display_mode = 'WINDOW' #'NONE'
	render_scene.render.pixel_filter_type = 'CATMULLROM'
	render_scene.render.engine = 'BLENDER_RENDER'
	render_scene.display_settings.display_device = 'None'
	render_scene.render.image_settings.color_depth = '8'
	render_scene.render.image_settings.color_mode = 'RGBA'
	render_scene.render.image_settings.file_format = 'TARGA_RAW'
	
	# smallIcon and icon20
	#create new camera
	cam = bpy.data.cameras.new('UI_small_cam')
	cam.clip_start = 0.1
	cam.clip_end = 200.0
	cam.type = 'ORTHO'
	cam.ortho_scale = 4
	cam_obj = bpy.data.objects.new('UI_small_cam', cam)
	cam_obj.location = Vector((0.0, 100.0, 5.5))
	cam_obj.rotation_mode = 'XYZ'
	cam_obj.rotation_euler = Euler((pi/2, 0.0, pi), 'XYZ')
	cam_obj.update_tag({'OBJECT', 'DATA'})

	#create lamps
	lamp1 = bpy.data.lamps.new('UI_small_sun', 'SUN')
	lamp1.energy = 0.4
	lamp1.use_specular = True
	lamp1.use_diffuse = True
	lamp1_obj = bpy.data.objects.new('UI_small_sun', lamp1)
	lamp1_obj.location = Vector((8.51651, 3.97130, 6.4202))
	lamp1_obj.rotation_mode = 'XYZ'
	lamp1_obj.rotation_euler = Euler((0.0, pi/6, pi*7/36), 'XYZ')
	lamp1_obj.update_tag({'OBJECT', 'DATA', 'TIME'})

	lamp2 = bpy.data.lamps.new('UI_small_area', 'AREA')
	lamp2.energy = 0.93
	lamp2.distance = 25.0
	lamp2.gamma = 0.0
	lamp2.size = 1.0
	lamp2.use_specular = False
	lamp2.use_diffuse = True
	lamp2_obj = bpy.data.objects.new('UI_small_area', lamp2)
	lamp2_obj.location = Vector((-100.0, 100.0, 2.3))
	lamp2_obj.rotation_mode = 'XYZ'
	lamp2_obj.rotation_euler = Euler((pi/2, 0.0, pi*5/4), 'XYZ')

	render_scene.objects.link(cam_obj)
	render_scene.objects.link(lamp1_obj)
	render_scene.objects.link(lamp2_obj)

	render_scene.camera = cam_obj
	
	return {"cam": cam_obj, "lamp1": lamp1_obj, "lamp2": lamp2_obj}
	
def clear_render_scene(render_scene):
	scene_objects = TF_base_functions.export_settings["render_scene_objects"]
	
	cam = scene_objects["cam"].data
	lamp1 = scene_objects["lamp1"].data
	lamp2 = scene_objects["lamp2"].data
	
	render_scene.objects.unlink(scene_objects["cam"])
	render_scene.objects.unlink(scene_objects["lamp1"])
	render_scene.objects.unlink(scene_objects["lamp2"])
	
	bpy.data.objects.remove(scene_objects["cam"])
	bpy.data.objects.remove(scene_objects["lamp1"])
	bpy.data.objects.remove(scene_objects["lamp2"])
	
	bpy.data.cameras.remove(cam)
	bpy.data.lamps.remove(lamp1)
	bpy.data.lamps.remove(lamp2)

def create_UI_icons(objects, models, errors):
	render_scene = TF_base_functions.export_settings["render_scene"]
	for obj in objects:
		render_scene.objects.link(obj)
	
	scene_objects = TF_base_functions.export_settings["render_scene_objects"]
	
	render_scene.update()
	
	stageImages = {}
	track_image = None
	road_image = None

	for model in models:
		data = {
			"modelData": model["data"],
			"icon": ("ui/models/" + model["model_type"], model["name"]),
			"smallIcon": ("ui/models_small/" + model["model_type"], model["name"]),
			"icon20": ("ui/models_20/" + model["model_type"], model["name"]),
			"iconHeight": 96,
			"camOffsetZ": 0,
			"stageImage": "",
		}
		TF_base_functions.call_plugin("create_ui_icon", data, errors)

		BB = model["BB"]
		if "boundingInfo" in model["data"]:
			BB = model["data"]["boundingInfo"]

		cam_size = round((BB["bbMax"][0]-BB["bbMin"][0])*4+0.5, 0)/4
		render_scene.render.resolution_y = data["iconHeight"]*4
		render_scene.render.resolution_x = int(cam_size*32)
		scene_objects["cam"].data.ortho_scale = cam_size
		scene_objects["cam"].location = Vector(((BB["bbMax"][0]+BB["bbMin"][0])/2 + model["object"].location.x, 100.0, data["iconHeight"]/16+data["camOffsetZ"]))
		
		render_scene.update()

		script_basepath = __file__.replace("UI_util.py", "images/")
		
		#render images
		if data["smallIcon"][0] != "" or data["icon20"][0] != "":
			filenames = {}
			renderImage = TF_base_functions.export_settings["override"]
			for iconType in ("icon", "smallIcon", "icon20"):
				if data[iconType][0] != "":
					if TF_base_functions.export_settings["dir_structure"]:
						filepath = TF_base_functions.export_settings["basepath"] + "textures/" + data[iconType][0]
					else:
						filepath = TF_base_functions.export_settings["basepath"]

					filenames[iconType] = filepath + data[iconType][1] + ".tga"
					if not exists(filepath):
						makedirs(filepath)
					
					renderImage |= (not isfile(filenames[iconType]))
		else:
			renderImage = False

		if renderImage:
			if data["icon"][0] != "":
				filename = filenames["icon"]
			elif data["smallIcon"][0] != "":
				filename = filenames["smallIcon"]
			elif data["icon20"][0] != "":
				filename = filenames["icon20"]
			
			images = []
			
			render_scene.render.filepath = filename
			bpy.ops.render.render(write_still = True, scene = render_scene.name)

			render_image = load_image(filename)
			if render_image:
				images.append(render_image.name)

			filename = ""
			if data["smallIcon"][0] != "":
				filename = filenames["smallIcon"]
			elif data["icon20"][0] != "":
				filename = filenames["icon20"]
			
			if filename != "":
				# scale render image
				if render_image:
					render_image.scale(int(cam_size*8), data["iconHeight"])

				# load background image
				if data["stageImage"] not in stageImages and data["stageImage"] != "":
					stageImages[data["stageImage"]] = load_image(script_basepath + "/%s_small.png" % data["stageImage"])

				# combine render image and background image
				if render_image and data["stageImage"] in stageImages and stageImages[data["stageImage"]]:
					render_image = image_util.combine_images(render_image, stageImages[data["stageImage"]], [0, 0], 'combined')
					images.append(render_image.name)

				# save icon
				if render_image:
					render_image.save_render(filename, scene=render_scene)
				
				if data["icon20"][0] != "":
					filename = filenames["icon20"]
					if render_image:
						render_image.scale(int(cam_size*8/(data["iconHeight"]/20)), 20)
						render_image.save_render(filename, scene=render_scene)

			# delete created images
			for image in images:
				if image in bpy.data.images:
					bpy.data.images[image].user_clear()
					bpy.data.images.remove(bpy.data.images[image])
			
	# delete background images
	for image in stageImages.values():
		if image:
			image.user_clear()
			bpy.data.images.remove(image)

	#revert previous settings
	for obj in objects:
		render_scene.objects.unlink(obj)